unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AppEvnts,DirectXGraphics, DXGUtils,math,d3dx8;

type
  TD3DFrm = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    D3D:iDirect3D8;//  
    D3DD:iDirect3DDevice8;//  
    VB:iDirect3DVertexBuffer8;//  
    Tex:iDirect3DTexture8;//  
    T1,T2:integer;
    function InitD3D:HResult;// Direct3D
    function InitVertexBuffer:HResult;//    
    function RenderScene:HResult;// 
    function InitTexture:HRESULT;//   
    procedure CleanUp;// 
  public
    { Public declarations }
  end;

// 
type
  TCUSTOMVERTEX = packed record
    X,Y,Z:Single;//  
    U, V : Single;//  
  end;

const
  D3DFVF_CUSTOMVERTEX=D3DFVF_XYZ or D3DFVF_TEX1;//    

var
  D3DFrm: TD3DFrm;
  Verts:array[0..5] of TCUSTOMVERTEX;//    
  Angle:Single;// 

implementation

{$R *.DFM}

procedure TD3DFrm.FormCreate(Sender: TObject);
var
   HRet:HResult;
begin
   Randomize;
   // Direct3D
   HRet:=InitD3D;
   if FAILED(HRet) then
   begin
      CleanUp;
      Close;
   end;
   //    
   HRet:=InitVertexBuffer;
   if FAILED(HRet) then
   begin
      CleanUp;
      Close;
   end;
   // 
   HRet:=InitTexture;
   if FAILED(HRet) then
   begin
      CleanUp;
      Close;
   end;

   Angle:=0;

   T1:=GetTickCount;
end;

procedure TD3DFrm.FormDestroy(Sender: TObject);
begin
   CleanUp;
end;

procedure TD3DFrm.CleanUp;
begin
   if Assigned(VB) then
   begin
      VB._Release;
      VB:=nil;
   end;
   if Assigned(Tex) then
   begin
      Tex._Release;
      Tex:=nil;
   end;
   if Assigned(D3DD) then
   begin
      D3DD._Release;
      D3DD:=nil;
   end;
   if Assigned(D3D) then
   begin
      D3D._Release;
      D3D:=nil;
   end;
end;

function TD3DFrm.InitD3D;
var
   D3DDM:TD3DDisplayMode;
   D3DPP:TD3DPresent_Parameters;
   HRet:HResult;
begin
   D3D:=Direct3DCreate8(D3D_SDK_Version);//   

   //    
   HRet:=D3D.GetAdapterDisplayMode(D3DAdapter_Default,D3DDM);
   if FAILED(HRet) then
   begin
      CleanUp;
   end;

   ZeroMemory(@D3DPP,SizeOf(D3DPP));// 
   With D3DPP do
   begin
      Windowed:=True;//  
      SwapEffect:=D3DSWAPEFFECT_DISCARD;//  
      BackBufferFormat:=D3DDM.Format;//  
      EnableAutoDepthStencil:=True;//  Z-
      AutoDepthStencilFormat:=D3DFMT_D16;// Z-
   end;

   Result:=D3D.CreateDevice(D3DAdapter_Default,// 
                           D3DDEVTYPE_HAL,// 3D Accelerator
                           Handle,// 
                           D3DCREATE_SOFTWARE_VERTEXPROCESSING,//  CPU
                           D3DPP,// 
                           D3DD);// 
end;

function TD3DFrm.InitVertexBuffer;
var
   HRet:HResult;
   PVerts:PByte;
begin
   //  
   Verts[0].X:=1;
   Verts[0].Y:=0;
   Verts[0].Z:=-1;
   Verts[0].U:=1;
   Verts[0].V:=0;

   Verts[1].X:=-1;
   Verts[1].Y:=0;
   Verts[1].Z:=-1;
   Verts[1].U:=0;
   Verts[1].V:=0;

   Verts[2].X:=-1;
   Verts[2].Y:=0;
   Verts[2].Z:=1;
   Verts[2].U:=0;
   Verts[2].V:=1;

   Verts[3].X:=-1;
   Verts[3].Y:=0;
   Verts[3].Z:=1;
   Verts[3].U:=0;
   Verts[3].V:=1;

   Verts[4].X:=1;
   Verts[4].Y:=0;
   Verts[4].Z:=1;
   Verts[4].U:=1;
   Verts[4].V:=1;

   Verts[5].X:=1;
   Verts[5].Y:=0;
   Verts[5].Z:=-1;
   Verts[5].U:=1;
   Verts[5].V:=0;

   //  
   HRet:=D3DD.CreateVertexBuffer(SizeOf(Verts),//   
                          D3DUSAGE_WRITEONLY,//    
                          D3DFVF_CUSTOMVERTEX,//  
                          D3DPOOL_DEFAULT,//     
                          VB);//  

   if FAILED(HRet) then
   begin
      CleanUp;
   end;
   // 
   VB.Lock(0,SizeOf(verts),PVerts,0);
   //  
   move(Verts,PVerts^,SizeOf(Verts));
   // 
   VB.Unlock;
   //        
   D3DD.SetStreamSource(0,VB,SizeOf(TCUSTOMVERTEX));
   //  
   Result:=D3DD.SetVertexShader(D3DFVF_CUSTOMVERTEX);
end;

function TD3DFrm.RenderScene;
var
  matView,matProj:TD3DMatrix;//   
  matRotateY:TD3DMatrix;// 
begin
   //     Z
   SetRotateYMatrix(matRotateY, Angle);
   //   
   D3DD.SetTransform(D3DTS_WORLD,matRotateY);
   //  
   SetViewMatrix(matView,D3DVector(1,2,1),D3DVector(0,0,0),D3DVector(0,1,0));
   //   
   D3DD.SetTransform(D3DTS_VIEW,matView);
   //  
   SetProjectionMatrix(matProj,1,1,1,40);
   //   
   D3DD.SetTransform(D3DTS_PROJECTION,matProj);
   // 
   D3DD.Clear(0,nil,D3DCLEAR_Target or D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(255,255,255),1,0);
   // 
   D3DD.BeginScene;

   D3DD.SetTextureStageState(0, D3DTSS_COLOROP, D3DTA_TEXTURE);

   //   
   D3DD.SetTextureStageState(0,D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   D3DD.SetTextureStageState(0,D3DTSS_MINFILTER, D3DTEXF_LINEAR);

   //    
   D3DD.SetTexture(0,Tex);

   //    
   D3DD.DrawPrimitive(D3DPT_TRIANGLELIST, 0,2);

   // 
   D3DD.EndScene;
   //  
   Result:=D3DD.Present(nil,nil,0,nil);
end;

function TD3DFrm.InitTexture:HResult;
begin
   Result:=D3DXCreateTextureFromFile(D3DD,// 
                               'Wood.bmp',// 
                               Tex);// 
end;

procedure TD3DFrm.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
   HRet:HResult;
begin
   HRet:=RenderScene;
   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'RenderScene',0);
   end;

   if T2-T1>10 then
   begin
      Angle:=Angle+0.05;
      
      T1:=GetTickCount;
   end;
   T2:=GetTickCount;
   done:=false;
end;

end.
